#!/usr/bin/perl -w
# 
# Copyright (c) BIO_MARK 2021
# Writer:         Dengdj <dengdj@biomarker.com.cn>
# Program Date:   2021
# Modifier:       Dengdj <dengdj@biomarker.com.cn>
# Last Modified:  2021
my $ver="1.2";

use strict;
use Getopt::Long;
use Data::Dumper;
use FindBin qw($Bin $Script);
use File::Basename qw(basename dirname);

#####################

my %opts;
GetOptions(\%opts,"f=s","b=s","g=s","o=s","h" );

#&help()if(defined $opts{h});
if(!defined($opts{f}) || !defined($opts{b}) || !defined($opts{g}) || !defined($opts{o}) || defined($opts{h}))
{
	print <<"	Usage End.";
	Description:
		
		Version: $ver
		   v1.2: change to use umi gene file.
		   v1.3: add head.

	Usage:

		-f           features.tsv file                          <infile>     must be given
		-b           barcodes.tsv file                          <infile>     must be given
		-g           umi gene file                              <infile>     must be given
		-o           result file                                <outfile>    must be given
		-h           Help document

	Usage End.

	exit;
}

###############Time
my $Time_Start;
$Time_Start = sub_format_datetime(localtime(time()));
&show_log("Start Time :[$Time_Start]");
################
$| = 1 ;
## get parameters
my $featuresfile = $opts{f} ;
my $barcodefile = $opts{b} ;
my $genefile = $opts{g} ;
my $outfile = $opts{o} ;

## get barcode info
my %hbc = ();
my $barcode_num = &get_barcode_info($barcodefile, \%hbc);

## get gene info
my %hgene = ();
my $gene_num = &get_gene_features($featuresfile, \%hgene);

## get gene umi info
my %hgene_bc = ();
&get_gene_umi_info($genefile, \%hgene_bc);

## generate matrix file
&generate_matrix_file(\%hgene_bc, \%hbc, \%hgene, $outfile, $barcode_num, $gene_num);

###############Time
my $Time_End;
$Time_End = sub_format_datetime(localtime(time()));
&show_log("End Time :[$Time_End]");

###############Subs
sub sub_format_datetime {#Time calculation subroutine
    my($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = @_;
	$wday = $yday = $isdst = 0;
    sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $day, $hour, $min, $sec);
}

sub ABSOLUTE_DIR
{ #$pavfile=&ABSOLUTE_DIR($pavfile);
	my $cur_dir=`pwd`;
	$cur_dir =~ s/\n$//;
	my ($in)=@_;
	my $return="";
	
	if(-f $in)
	{
		my $dir=dirname($in);
		my $file=basename($in);
		chdir $dir;$dir=`pwd`;
		$dir =~ s/\n$// ;
		$return="$dir/$file";
	}
	elsif(-d $in)
	{
		chdir $in;$return=`pwd`;
		$return =~ s/\n$// ;
	}
	else
	{
		warn "Warning just for file and dir [$in]\n";
		exit;
	}
	
	chdir $cur_dir;
	return $return;
}

# &show_log("txt")
sub show_log()
{
	my ($txt) = @_ ;
	my $time = time();
	my $Time = &sub_format_datetime(localtime($time));
	print "$Time:\t$txt\n" ;
	return ($time) ;
}

#&run_or_die($cmd);
sub run_or_die()
{
	my ($cmd) = @_ ;
	my $start_time = &show_log($cmd);
	my $flag = system($cmd) ;
	if ($flag != 0){
		my $end_time = &show_log("Error: command fail: $cmd");
		&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
		exit(1);
	}
	my $end_time = &show_log("done.");
	&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
	return ;
}

## qsub
sub qsub()
{
	my ($shfile, $maxproc) = @_ ;
	my $dir = dirname($shfile) ;
	my $cmd = "cd $dir && sh /share/nas2/genome/bmksoft/tool/qsub_sge_plus/v1.0/qsub_sge.plus.sh --maxproc $maxproc --reqsub --independent $shfile" ;
	&run_or_die($cmd);

	return ;
}

## qsub_mult($shfile, $max_proc, $job_num)
sub qsub_mult()
{
	my ($shfile, $max_proc, $job_num) = @_ ;
	if ($job_num > 500){
		my @shfiles = &cut_shfile($shfile);
		for my $file (@shfiles){
			&qsub($file, $max_proc);
		}
	}
	else{
		&qsub($shfile, $max_proc) ;
	}
}

#my @shfiles = &cut_shfile($shfile);
sub cut_shfile()
{
	my ($file) = @_ ;
	my @files = ();
	my $num = 0 ;
	open (IN, $file) || die "Can't open $file, $!\n" ;
	(my $outprefix = $file) =~ s/.sh$// ;
	while (<IN>){
		chomp ;
		if ($num % 500 == 0){
			close(OUT);
			my $outfile = "$outprefix.sub_".(int($num/500)+1).".sh" ;
			push @files, $outfile ;
			open (OUT, ">$outfile") || die "Can't creat $outfile, $!\n" ;
		}
		print OUT $_, "\n" ;
		$num ++ ;
	}
	close(IN);
	close(OUT);

	return @files ;
}

sub sub_normal_dis_ran(){#$average,$standard_deviation
        my ($aver,$stand_dev) = @_ ;
        my $ran1 = rand() ;
        my $ran2 = rand() ;
        my $y = ((-2*log($ran1))**(0.5))*sin(2*3.1415926535*$ran2) ;
        return ($aver + $stand_dev*$y) ;
}

#my $barcode_num = &get_barcode_info($barcodefile, \%hbc);
sub get_barcode_info()
{
    my ($bcfile, $ahbc) = @_ ;

    open (IN, $bcfile) || die "$bcfile, $!\n" ;
    my $index = 0 ;
    while(<IN>){
        chomp ;
        next if (m/^\#|^\s*$/);
        $index++ ;
        my ($bc) = split ;
        $ahbc->{$bc} = $index ;
    }
    close(IN);

    return($index);
}

#my $gene_num = &get_gene_features($featuresfile, \%hgene);
sub get_gene_features()
{
    my ($infile, $ahgene) = @_ ;

    open (IN, $infile) || die "$infile, $!\n" ;
    my $index = 0 ;
    while(<IN>){
        chomp ;
        next if (m/^\#|^\s*$/);
        $index++ ;
        my ($id) = split ;
        $ahgene->{$id} = $index ;
    }
    close(IN);

    return($index);
}

#&get_gene_umi_info($genefile, \%hgene_bc);
sub get_gene_umi_info()
{
    my ($genefile, $ahgene_bc) = @_ ;

    open (IN, $genefile) || die "$genefile, $!\n" ;
    while(<IN>){
        chomp ;
        next if (m/^\s*$|^\#/);
        my ($bc, $umi, $gene) = split ;
        $ahgene_bc->{$gene}{$bc}++ ;
    }
    close(IN);

    return ;
}

#&generate_matrix_file(\%hgene_bc, \%hbc, \%hgene, $outfile, $barcode_num, $gene_num);
sub generate_matrix_file()
{
    my ($ahgene_bc, $ahbc, $ahgene, $outfile, $barcode_num, $gene_num) = @_ ;

    open (FIL, ">$outfile.filt") || die "$outfile.filt, $!\n" ;
    my $out = "" ;
    my $matrix_line = 0 ;
    for my $gene (keys %{$ahgene_bc}){
        next if (!defined $ahgene->{$gene}) ;
        my $gene_index = $ahgene->{$gene} ;
        for my $bc (keys %{$ahgene_bc->{$gene}}){
            if (!defined $ahbc->{$bc}){
                my $num = $ahgene_bc->{$gene}{$bc} ;
                print FIL "$gene\t$bc\t$num\n" ;
            }
            else{
                my $bc_index = $ahbc->{$bc} ;
                my $num = $ahgene_bc->{$gene}{$bc} ;
                #print OUT "$gene_index\t$bc_index\t$num\n" ;
                $out .= "$gene_index\t$bc_index\t$num\n" ;
                $matrix_line++ ;
            }
        }
    }
    close(FIL) ;
    open (OUT, ">$outfile") || die "$outfile, $!\n" ;
    print OUT "%%MatrixMarket matrix coordinate integer general\n" ;
    print OUT "%\n" ;
    print OUT "$gene_num\t$barcode_num\t$matrix_line\n" ;
    print OUT "$out" ;
    close(OUT) ;

    return ;
}

