#!/usr/bin/perl -w
use strict ;

if (@ARGV != 3){
    print "perl $0 fqfile infile outfile\n" ;
    exit(1) ;
}

my ($fqfile, $infile, $outfile) = @ARGV ;
$|=1 ;

if ($fqfile !~ /\.fq$|\.fastq$|\.fq.gz$|\.fastq.gz$/){
    print "reads file must be end with \".fq, .fastq, .fq.gz, .fastq.gz\": $fqfile\n" ;
    exit(1);
}

if ($fqfile =~ m/\.fq.gz$|\.fastq.gz$/i){
    open (SEQ, "gunzip -c $fqfile |") || die "$fqfile, $!\n" ;
}
elsif($fqfile =~ m/\.fq$|\.fastq$/i){
    open (SEQ, $fqfile) || die "$fqfile, $!\n" ;
}

my $umi_len = 12 ;
my %humi_pos = (
    'bc1' => 57,
    'bc2' => 34,
    'bc3' => 12,
);
my %href_pos = (
	'bc1'   => 0 ,
	'bc2'   => 23 ,
	'bc3'   => 45 ,
);

open (IN, $infile) || die "$infile, $!\n" ;
my %hbc = ();
my %hbc_umi = ();
my %hsingle = ();
my %hbc_stat = ();
my $total = 0 ;
open (OUT, ">$outfile.umi") || die "$outfile.umi, $!\n" ;
while(<IN>){
    chomp ;
    my ($id, $type, $posi) = split ;
    $total++ ;
    next if ($type =~ /null/i);
    my $seq = &get_next_seq($id);
    if ($type =~ m/bc1/ && $type =~ m/bc2/ && $type =~ m/bc3/){
        $hbc{$type}++ ;
    }
    my @poses = split /\-/, $posi ;
    my @types = split /\-/, $type ;
    my @bcs = ();
    my $umi_seq = "" ;
    my (@bc_poses, $final_umi_pos) = ();
    for (my $i=0; $i<@types; $i++){
        if ($types[$i] =~ m/(bc\d)\_\d+/){
            $hsingle{$1}++ ;
            push @bcs, $1 ;
            my ($ref_pos, $map_pos) = split /,/, $poses[$i] ;
            my $umi_pos = $humi_pos{$1} + ($map_pos - $ref_pos) + $href_pos{$1} ;
            $umi_seq = substr($seq, $umi_pos, $umi_len);
            $final_umi_pos = $umi_pos ;
            my $bc_start_pos = $href_pos{$1} + ($map_pos - $ref_pos) ;
            $bc_start_pos = 0 if ($bc_start_pos < 0) ;
            push @bc_poses, $bc_start_pos ;
        }
    }
    my $bc_type = join('-', @bcs) ;
    $hbc_stat{$bc_type}++ ;
    my $bc_pos = join('-', @bc_poses) ;

    $hbc_umi{$type}{$umi_seq}++ ;
    print OUT "$id\t$type\t$posi\t$umi_seq\t$final_umi_pos\n" ;
    #print POS "$id\t$bc_type\t$bc_pos\t$final_umi_pos\n" ;
}
close(OUT);
close(SEQ);
close(IN);

open (OUT, ">$outfile.full_stat") || die "$outfile.full_stat, $!\n" ;
#print OUT "#Type\tReadsNum\tPercent(%)\n" ;
print OUT "#Type\tReadsNum\tUMI\n" ;
for my $type (sort {$hbc{$b}<=>$hbc{$a}} keys %hbc){
    #my $rate = int(($hbc{$type}/$total)*10000)/100 ;
    my $umi = scalar(keys %{$hbc_umi{$type}}) ;
    print OUT "$type\t", $hbc{$type}, "\t", $umi, "\n" ;
}
close(OUT);

open (OUT, ">$outfile.bc_stat") || die "$outfile.bc_stat, $!\n" ;
print OUT "#Type\tReadsNum\tPercent(%)\n" ;
for my $type (sort keys %hbc_stat){
    my $num = $hbc_stat{$type} ;
    my $rate = int(($num/$total)*10000)/100 ;
    print OUT "$type\t$num\t$rate\n" ;
}
print OUT "========= single stat =========\n" ;
for my $type (sort keys %hsingle){
    my $num = $hsingle{$type} ;
    my $rate = int(($num/$total)*10000)/100 ;
    print OUT "$type\t$num\t$rate\n" ;
}
close(OUT) ;

print `cut -f 1 $outfile.full_stat | grep -v "#" > $outfile.barcodes.tsv` ;

#my $seq = &get_next_seq($id);
sub get_next_seq()
{
    my ($id) = @_ ;

    while(1){
        my $readid = <SEQ> ;
        my $seq = <SEQ> ;
        my $readid2 = <SEQ> ;
        my $qual = <SEQ> ;
        $readid = (split /\s+/, $readid)[0] ;
        $readid =~ s/^\@// ;
        $readid =~ s/\/1$// ;
        $seq =~ s/\n//g ;
        if ($readid eq $id){
            return($seq);
        }
    }
    print "can't find reads id in seq file, please check: $id\n" ;
    exit(1);

    return ;
}

