#!/usr/bin/perl -w
# 
# Copyright (c) BIO_MARK 2021
# Writer:         Dengdj <dengdj@biomarker.com.cn>
# Program Date:   2021
# Modifier:       Dengdj <dengdj@biomarker.com.cn>
# Last Modified:  2021
my $ver="1.2";

use strict;
use Getopt::Long;
use Data::Dumper;
use FindBin qw($Bin $Script);
use File::Basename qw(basename dirname);

#####################

my %opts;
GetOptions(\%opts,"i=s","o=s","k=s","m=s","h" );

#&help()if(defined $opts{h});
if(!defined($opts{i}) || !defined($opts{k}) || !defined($opts{o}) || defined($opts{h}))
{
	print <<"	Usage End.";
	Description:
		
		Version: $ver

	Usage:

		-i           reads file                              <infile>     must be given
		-k           kmer file                               <infile>     must be given
		-o           result file                             <outfile>    must be given
		-m           min kmer number                         [int]        optional[3]  
		-h           Help document

	Usage End.

	exit;
}

###############Time
my $Time_Start;
$Time_Start = sub_format_datetime(localtime(time()));
&show_log("Start Time :[$Time_Start]");
################
$| = 1 ;
## get parameters
my $readsfile = $opts{i} ;
my $kmerfile = $opts{k} ;
my $outfile = $opts{o} ;
my $minkmernum = defined $opts{m} ? $opts{m} : 3 ;

## judge barcode by kmer
if ($readsfile =~ m/\.fa$|\.fasta$/i){
    my $cmd = "perl $Bin/judge_barcode_by_kmer_fa_v1.3.pl $readsfile $kmerfile $outfile.out 8" ;
    &run_or_die($cmd) ;
}
elsif($readsfile =~ m/\.fq$|\.fastq$|\.fq.gz$|\.fastq.gz/i){
    my $cmd = "perl $Bin/judge_barcode_by_kmer_v1.4.pl $readsfile $kmerfile $outfile.out 8" ;
    &run_or_die($cmd) ;
}
else{
    &show_log("Error: reads file mast end with .fa/.fasta/.fq/.fastq/.fq.gz/.fastq.gz");
    exit(1);
}

## get barcode type
my $cmd = "perl $Bin/get_barcode_type_v1.3.new_bc.pl -i $outfile.out -o $outfile.type -m $minkmernum" ;
&run_or_die($cmd) ;

## stat barcode type
$cmd = "perl $Bin/barcode_stat_v1.5.new_bc.pl $readsfile $outfile.type $outfile.type" ;
&run_or_die($cmd) ;

## get dist
$cmd = "perl $Bin/bc_dist_stat_umi.pl $outfile.type.full_stat $outfile.type.bc_dist" ;
&run_or_die($cmd) ;

## get bc umi
$cmd = "perl $Bin/barcode_umi_stat.pl -i $outfile.type.umi -o $outfile.bc_umi_read.tsv" ;
&run_or_die($cmd) ;

###############Time
my $Time_End;
$Time_End = sub_format_datetime(localtime(time()));
&show_log("End Time :[$Time_End]");

###############Subs
sub sub_format_datetime {#Time calculation subroutine
    my($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = @_;
	$wday = $yday = $isdst = 0;
    sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $day, $hour, $min, $sec);
}

sub ABSOLUTE_DIR
{ #$pavfile=&ABSOLUTE_DIR($pavfile);
	my $cur_dir=`pwd`;
	$cur_dir =~ s/\n$//;
	my ($in)=@_;
	my $return="";
	
	if(-f $in)
	{
		my $dir=dirname($in);
		my $file=basename($in);
		chdir $dir;$dir=`pwd`;
		$dir =~ s/\n$// ;
		$return="$dir/$file";
	}
	elsif(-d $in)
	{
		chdir $in;$return=`pwd`;
		$return =~ s/\n$// ;
	}
	else
	{
		warn "Warning just for file and dir [$in]\n";
		exit;
	}
	
	chdir $cur_dir;
	return $return;
}

# &show_log("txt")
sub show_log()
{
	my ($txt) = @_ ;
	my $time = time();
	my $Time = &sub_format_datetime(localtime($time));
	print "$Time:\t$txt\n" ;
	return ($time) ;
}

#&run_or_die($cmd);
sub run_or_die()
{
	my ($cmd) = @_ ;
	my $start_time = &show_log($cmd);
	my $flag = system($cmd) ;
	if ($flag != 0){
		my $end_time = &show_log("Error: command fail: $cmd");
		&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
		exit(1);
	}
	my $end_time = &show_log("done.");
	&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
	return ;
}

## qsub
sub qsub()
{
	my ($shfile, $maxproc) = @_ ;
	my $dir = dirname($shfile) ;
	my $cmd = "cd $dir && sh /share/nas2/genome/bmksoft/tool/qsub_sge_plus/v1.0/qsub_sge.plus.sh --maxproc $maxproc --reqsub --independent $shfile" ;
	&run_or_die($cmd);

	return ;
}

## qsub_mult($shfile, $max_proc, $job_num)
sub qsub_mult()
{
	my ($shfile, $max_proc, $job_num) = @_ ;
	if ($job_num > 500){
		my @shfiles = &cut_shfile($shfile);
		for my $file (@shfiles){
			&qsub($file, $max_proc);
		}
	}
	else{
		&qsub($shfile, $max_proc) ;
	}
}

#my @shfiles = &cut_shfile($shfile);
sub cut_shfile()
{
	my ($file) = @_ ;
	my @files = ();
	my $num = 0 ;
	open (IN, $file) || die "Can't open $file, $!\n" ;
	(my $outprefix = $file) =~ s/.sh$// ;
	while (<IN>){
		chomp ;
		if ($num % 500 == 0){
			close(OUT);
			my $outfile = "$outprefix.sub_".(int($num/500)+1).".sh" ;
			push @files, $outfile ;
			open (OUT, ">$outfile") || die "Can't creat $outfile, $!\n" ;
		}
		print OUT $_, "\n" ;
		$num ++ ;
	}
	close(IN);
	close(OUT);

	return @files ;
}

