#!/usr/bin/perl -w
# 
# Copyright (c) BIO_MARK 2021
# Writer:         Dengdj <dengdj@biomarker.com.cn>
# Program Date:   2021
# Modifier:       Dengdj <dengdj@biomarker.com.cn>
# Last Modified:  2021
my $ver="1.0";

use strict;
use Getopt::Long;
use Data::Dumper;
use FindBin qw($Bin $Script);
use File::Basename qw(basename dirname);

#####################

my %opts;
GetOptions(\%opts,"i=s","o=s","n=s","h" );

#&help()if(defined $opts{h});
if(!defined($opts{i}) || !defined($opts{o}) || defined($opts{h}))
{
	print <<"	Usage End.";
	Description:
		
		Version: $ver

	Usage:

		-i           matrix file dir                        <infile>     must be given
		-o           result dir                             <outdir>     must be given
		-n           remain barcode number                  [int]        optinal [10000]
		-h           Help document

	Usage End.

	exit;
}

###############Time
my $Time_Start;
$Time_Start = sub_format_datetime(localtime(time()));
&show_log("Start Time :[$Time_Start]");
################
$| = 1 ;
## get parameters
my $indir = $opts{i} ;
my $outdir = $opts{o} ;
my $min_num = defined $opts{n} ? $opts{n} : 10000 ;
mkdir $outdir if (!-d $outdir) ;

## check file
my $bcfile = glob("$indir/*barcodes.tsv.gz") ;
if (!-f $bcfile){
    &show_log("barcode file not exist, $bcfile");
    die ;
}
my $mtfile = glob("$indir/*matrix.mtx.gz") ;
if (!-f $mtfile){
    &show_log("matrix file not exist, $mtfile");
    die ;
}

## get barcode info
my %hbc = ();
&get_barcode_info($bcfile, \%hbc);

## get matrix
&get_matrix($mtfile, \%hbc, $outdir, $min_num);

my $ftfile = glob("$indir/features.tsv.gz") ;
if (-f $ftfile){
    my $cmd = "cp $ftfile $outdir/" ;
    &run_or_die($cmd) ;
}


###############Time
my $Time_End;
$Time_End = sub_format_datetime(localtime(time()));
&show_log("End Time :[$Time_End]");

###############Subs
sub sub_format_datetime {#Time calculation subroutine
    my($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = @_;
	$wday = $yday = $isdst = 0;
    sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $day, $hour, $min, $sec);
}

sub ABSOLUTE_DIR
{ #$pavfile=&ABSOLUTE_DIR($pavfile);
	my $cur_dir=`pwd`;
	$cur_dir =~ s/\n$//;
	my ($in)=@_;
	my $return="";
	
	if(-f $in)
	{
		my $dir=dirname($in);
		my $file=basename($in);
		chdir $dir;$dir=`pwd`;
		$dir =~ s/\n$// ;
		$return="$dir/$file";
	}
	elsif(-d $in)
	{
		chdir $in;$return=`pwd`;
		$return =~ s/\n$// ;
	}
	else
	{
		warn "Warning just for file and dir [$in]\n";
		exit;
	}
	
	chdir $cur_dir;
	return $return;
}

# &show_log("txt")
sub show_log()
{
	my ($txt) = @_ ;
	my $time = time();
	my $Time = &sub_format_datetime(localtime($time));
	print "$Time:\t$txt\n" ;
	return ($time) ;
}

#&run_or_die($cmd);
sub run_or_die()
{
	my ($cmd) = @_ ;
	my $start_time = &show_log($cmd);
	my $flag = system($cmd) ;
	if ($flag != 0){
		my $end_time = &show_log("Error: command fail: $cmd");
		&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
		exit(1);
	}
	my $end_time = &show_log("done.");
	&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
	return ;
}

## qsub
sub qsub()
{
	my ($shfile, $maxproc) = @_ ;
	my $dir = dirname($shfile) ;
	my $cmd = "cd $dir && sh /share/nas2/genome/bmksoft/tool/qsub_sge_plus/v1.0/qsub_sge.plus.sh --maxproc $maxproc --reqsub --independent $shfile" ;
	&run_or_die($cmd);

	return ;
}

## qsub_mult($shfile, $max_proc, $job_num)
sub qsub_mult()
{
	my ($shfile, $max_proc, $job_num) = @_ ;
	if ($job_num > 500){
		my @shfiles = &cut_shfile($shfile);
		for my $file (@shfiles){
			&qsub($file, $max_proc);
		}
	}
	else{
		&qsub($shfile, $max_proc) ;
	}
}

#my @shfiles = &cut_shfile($shfile);
sub cut_shfile()
{
	my ($file) = @_ ;
	my @files = ();
	my $num = 0 ;
	open (IN, $file) || die "Can't open $file, $!\n" ;
	(my $outprefix = $file) =~ s/.sh$// ;
	while (<IN>){
		chomp ;
		if ($num % 500 == 0){
			close(OUT);
			my $outfile = "$outprefix.sub_".(int($num/500)+1).".sh" ;
			push @files, $outfile ;
			open (OUT, ">$outfile") || die "Can't creat $outfile, $!\n" ;
		}
		print OUT $_, "\n" ;
		$num ++ ;
	}
	close(IN);
	close(OUT);

	return @files ;
}

sub sub_normal_dis_ran(){#$average,$standard_deviation
        my ($aver,$stand_dev) = @_ ;
        my $ran1 = rand() ;
        my $ran2 = rand() ;
        my $y = ((-2*log($ran1))**(0.5))*sin(2*3.1415926535*$ran2) ;
        return ($aver + $stand_dev*$y) ;
}

#&get_barcode_info($bcfile, \%hbc);
sub get_barcode_info()
{
    my ($bcfile, $ahbc) = @_ ;

    open (IN, "gunzip -c $bcfile |") || die "$bcfile, $!\n" ;
    my $index = 0 ;
    while(<IN>){
        chomp ;
        my $bc = (split)[0] ;
        $index++ ;
        $ahbc->{$index} = $bc ;
    }
    close(IN);

    return ;
}

#&get_matrix($mtfile, \%hbc, $outdir, $min_num);
sub get_matrix()
{
    my ($mtfile, $ahbc, $outdir, $min_num) = @_ ;

    open (IN, "gunzip -c $mtfile|") || die "$mtfile, $!\n" ;
    my $head1 = <IN> ;
    my $head2 = <IN> ;
    my $head3 = <IN> ;
    my $gene_num = (split /\s+/, $head3)[0] ;
    my %hmt = ();
    my %hgene = ();
    while(<IN>){
        chomp ;
        my ($gene, $bc, $umi) = split ;
        $hgene{$bc}{$gene}++ ;
        $hmt{$bc}{$gene} = $umi ;
    }
    close(IN);

    my %hgene_num = ();
    for my $bc (keys %hgene){
        my $num = scalar(keys %{$hgene{$bc}}) ;
        $hgene_num{$bc} = $num ;
    }

    open (MT, "| gzip > $outdir/matrix.mtx.gz") || die "$outdir/matrix.mtx.gz, $!\n" ;
    open (BC, "| gzip > $outdir/barcodes.tsv.gz") || die "$outdir/barcodes.tsv.gz, $!\n" ;
    my $num = 0 ;
    my $str = "" ;
    my $line = 0 ;
    for my $bc (sort {$hgene_num{$b}<=>$hgene_num{$a}} keys %hgene_num){
        $num++ ;
        for my $gene (keys %{$hgene{$bc}}){
            my $umi = $hmt{$bc}{$gene} ;
            $str .= "$gene $num $umi\n" ;
            $line++ ;
        }
        print BC $ahbc->{$bc}, "\n" ;
        last if ($num >= $min_num) ;
    }
    print MT $head1, $head2 ;
    print MT "$gene_num $num $line\n" ;
    print MT "$str" ;
    close(MT);
    close(BC);

    return ;
}

