import os
import numpy as np
import argparse
from plotly import graph_objects as go
from cellranger.matrix import CountMatrix
from cellranger.cell_calling import find_nonambient_barcodes
from cellranger.io import open_maybe_gzip
from cellranger.webshim.data import compute_sort_order,compute_plot_segments
from cellranger.webshim.common import build_plot_data_dict

def parseArgs():
    parser = argparse.ArgumentParser('')
    parser.add_argument('--matrix', '-m', required=True, help='raw feature barcode matrix')
    parser.add_argument('--filtered-matrix', '-f', required=True, help='filtered feature barcode matrix')
    parser.add_argument('--outfile', '-o', default='rank_barcode_umi.html', help='output file')
    return parser.parse_args()

def readmatrix(matrixDir):
    matrix = CountMatrix.from_v3_mtx(matrixDir) 
    matrix.m = matrix.m.astype('int64')
    matrix.tocsc()
    return matrix

def barcode_umi_plot(counts_per_bc, raw_barcodes, cell_barcodes, outfile):
    srt_order = compute_sort_order(counts_per_bc, raw_barcodes, cell_barcodes)
    sorted_bc = raw_barcodes[srt_order]
    sorted_counts = counts_per_bc[srt_order]
    plot_segments = compute_plot_segments(sorted_bc, sorted_counts, cell_barcodes)
    plot_data = [build_plot_data_dict(i, sorted_counts) for i in plot_segments]
    layout = go.Layout(xaxis={'type':'log', 'title':'Barcodes'}, 
                       yaxis={'type':'log', 'title':'UMI counts'},
                       width=800, height=600)
    fig =  go.Figure(plot_data, layout=layout)
    fig.write_html(outfile, config= {'displaylogo': False})

if __name__=='__main__':
    args = parseArgs()
    outfile = args.outfile
    matrixDir = args.matrix
    filtMtDir = args.filtered_matrix
    matrix = readmatrix(matrixDir)
    filtered_matrix = readmatrix(filtMtDir)
    cell_barcodes = filtered_matrix.bcs[:]
    counts_per_bc = matrix.get_counts_per_bc()
    raw_barcodes = matrix.bcs[:]
    barcode_umi_plot(counts_per_bc, raw_barcodes, cell_barcodes, outfile)
    print('completed!\n')
