#!/usr/bin/perl -w
# 
# Copyright (c) BIO_MARK 2021
# Writer:         Dengdj <dengdj@biomarker.com.cn>
# Program Date:   2021
# Modifier:       Dengdj <dengdj@biomarker.com.cn>
# Last Modified:  2021
my $ver="1.0";

use strict;
use Getopt::Long;
use Data::Dumper;
use FindBin qw($Bin $Script);
use File::Basename qw(basename dirname);

#####################

my %opts;
GetOptions(\%opts,"i=s","g=s","o=s","h" );

#&help()if(defined $opts{h});
if(!defined($opts{i}) || !defined($opts{g}) || !defined($opts{o}) || defined($opts{h}))
{
	print <<"	Usage End.";
	Description:
		
		Version: $ver

	Usage:

		-i           reads mapping sam file                <infile>     must be given
		-g           gene gff file                         <infile>     must be given
		-o           result file                           <outfile>    must be given
		-h           Help document

	Usage End.

	exit;
}

###############Time
my $Time_Start;
$Time_Start = sub_format_datetime(localtime(time()));
&show_log("Start Time :[$Time_Start]");
################
$| = 1 ;
## get parameters
my $samfile = $opts{i} ;
my $gfffile = $opts{g} ;
my $outfile = $opts{o} ;

## get gene info
my %hgene = ();
&get_gene_info($gfffile, \%hgene);

## get reads mapping info
my %hmap = ();
&get_reads_mapping_info($samfile, \%hmap);

## get reads gene
my %hid = ();
&get_reads_gene(\%hgene, \%hmap, \%hid);

## out put
&out_put(\%hid, $outfile);

###############Time
my $Time_End;
$Time_End = sub_format_datetime(localtime(time()));
&show_log("End Time :[$Time_End]");

###############Subs
sub sub_format_datetime {#Time calculation subroutine
    my($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = @_;
	$wday = $yday = $isdst = 0;
    sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $day, $hour, $min, $sec);
}

sub ABSOLUTE_DIR
{ #$pavfile=&ABSOLUTE_DIR($pavfile);
	my $cur_dir=`pwd`;
	$cur_dir =~ s/\n$//;
	my ($in)=@_;
	my $return="";
	
	if(-f $in)
	{
		my $dir=dirname($in);
		my $file=basename($in);
		chdir $dir;$dir=`pwd`;
		$dir =~ s/\n$// ;
		$return="$dir/$file";
	}
	elsif(-d $in)
	{
		chdir $in;$return=`pwd`;
		$return =~ s/\n$// ;
	}
	else
	{
		warn "Warning just for file and dir [$in]\n";
		exit;
	}
	
	chdir $cur_dir;
	return $return;
}

# &show_log("txt")
sub show_log()
{
	my ($txt) = @_ ;
	my $time = time();
	my $Time = &sub_format_datetime(localtime($time));
	print "$Time:\t$txt\n" ;
	return ($time) ;
}

#&run_or_die($cmd);
sub run_or_die()
{
	my ($cmd) = @_ ;
	my $start_time = &show_log($cmd);
	my $flag = system($cmd) ;
	if ($flag != 0){
		my $end_time = &show_log("Error: command fail: $cmd");
		&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
		exit(1);
	}
	my $end_time = &show_log("done.");
	&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
	return ;
}

## qsub
sub qsub()
{
	my ($shfile, $maxproc) = @_ ;
	my $dir = dirname($shfile) ;
	my $cmd = "cd $dir && sh /share/nas2/genome/bmksoft/tool/qsub_sge_plus/v1.0/qsub_sge.plus.sh --maxproc $maxproc --reqsub --independent $shfile" ;
	&run_or_die($cmd);

	return ;
}

## qsub_mult($shfile, $max_proc, $job_num)
sub qsub_mult()
{
	my ($shfile, $max_proc, $job_num) = @_ ;
	if ($job_num > 500){
		my @shfiles = &cut_shfile($shfile);
		for my $file (@shfiles){
			&qsub($file, $max_proc);
		}
	}
	else{
		&qsub($shfile, $max_proc) ;
	}
}

#my @shfiles = &cut_shfile($shfile);
sub cut_shfile()
{
	my ($file) = @_ ;
	my @files = ();
	my $num = 0 ;
	open (IN, $file) || die "Can't open $file, $!\n" ;
	(my $outprefix = $file) =~ s/.sh$// ;
	while (<IN>){
		chomp ;
		if ($num % 500 == 0){
			close(OUT);
			my $outfile = "$outprefix.sub_".(int($num/500)+1).".sh" ;
			push @files, $outfile ;
			open (OUT, ">$outfile") || die "Can't creat $outfile, $!\n" ;
		}
		print OUT $_, "\n" ;
		$num ++ ;
	}
	close(IN);
	close(OUT);

	return @files ;
}

sub sub_normal_dis_ran(){#$average,$standard_deviation
        my ($aver,$stand_dev) = @_ ;
        my $ran1 = rand() ;
        my $ran2 = rand() ;
        my $y = ((-2*log($ran1))**(0.5))*sin(2*3.1415926535*$ran2) ;
        return ($aver + $stand_dev*$y) ;
}

#&get_gene_info($gfffile, \%hgene);
sub get_gene_info()
{
    my ($gfffile, $ahgene) = @_ ;

    open (IN, $gfffile) || die "$gfffile, $!\n" ;
    while(<IN>){
        chomp ;
        next if (m/^\#|^\s*$/);
        my ($chr, $soft, $type, $start, $end, undef, $ori, $offset, $info) = split ;
        if ($type eq 'gene'){
            my $id ;
            if ($info =~ m/ID=(.*?)\;/){
                $id = $1 ;
            }
            push @{$ahgene->{$chr}}, [$start, $end, $id] ;
        }
    }
    close(IN);

    return ;
}

#&get_reads_mapping_info($samfile, \%hmap);
sub get_reads_mapping_info()
{
    my ($samfile, $ahmap) = @_ ;

    open (IN, $samfile) || die "$samfile, $!\n" ;
    while(<IN>){
        chomp ;
        next if (m/^\@/);
        my ($id, $flag, $chr, $start, $mapQ, $cigar, undef, undef, undef, $seq, $qual) = split ;
        my $maplen = &get_maplen($cigar);
        my $end = $start + $maplen - 1 ;
        push @{$ahmap->{$chr}}, [$start, $end, $id] ;
    }
    close(IN);

    return ;
}

#my $maplen = &get_maplen($cigar);
sub get_maplen()
{
    my ($cigar) = @_ ;

    my $len = 0 ;
    while($cigar =~ m/(\d+)([a-zA-Z])/g){
        my ($num, $type) = ($1, $2) ;
        if ($type eq 'M' || $type eq 'N' || $type eq 'D' || $type eq '=' || $type eq 'X'){
            $len += $num ;
        }
    }

    return($len);
}

#&get_reads_gene(\%hgene, \%hmap, \%hid);
sub get_reads_gene()
{
    my ($ahgene, $ahmap, $ahid) = @_ ;

    open (OUT, ">$outfile") || die "$outfile, $!\n" ;
    for my $chr (keys %{$ahmap}){
        my @gene_region = sort {$a->[0]<=>$b->[0] || $a->[1]<=>$b->[1]} @{$ahgene->{$chr}} ;
        my @reads_region = sort {$a->[0]<=>$b->[0] || $a->[1]<=>$b->[1]} @{$ahmap->{$chr}} ;
        my $index = 0 ;
        for (my $i=0; $i<@reads_region; $i++){
            my ($r_start, $r_end, $r_id) = @{$reads_region[$i]} ;
            for (my $j=$index; $j<@gene_region; $j++){
                my ($g_start, $g_end, $g_id) = @{$gene_region[$j]} ;
                if ($r_end < $g_start){
                    last ;
                }
                elsif ($r_start <= $g_end){
                    push @{$ahid->{$r_id}}, $g_id ;
                }
                else{
                    $index = $j ;
                    next ;
                }
            }
        }
    }
    close(OUT) ;

    return ;
}

#&out_put(\%hid, $outfile);
sub out_put()
{
    my ($ahid, $outfile) = @_ ;

    open (OUT, ">$outfile") || die "$outfile, $!\n" ;
    for my $id (keys %{$ahid}){
        print OUT "$id" ;
        for (my $i=0; $i<@{$ahid->{$id}}; $i++){
            print OUT "\t", $ahid->{$id}[$i] ;
        }
        print OUT "\n" ;
    }
    close(OUT) ;

    return ;
}

