#!/usr/bin/perl -w
# 
# Copyright (c) BIO_MARK 2021
# Writer:         Dengdj <dengdj@biomarker.com.cn>
# Program Date:   2021
# Modifier:       Dengdj <dengdj@biomarker.com.cn>
# Last Modified:  2021
my $ver="1.5";

use strict;
use Getopt::Long;
use Data::Dumper;
use FindBin qw($Bin $Script);
use File::Basename qw(basename dirname);

#####################

my %opts;
GetOptions(\%opts,"u=s","g=s","o=s","h" );

#&help()if(defined $opts{h});
if(!defined($opts{u}) || !defined($opts{g}) || !defined($opts{o}) || defined($opts{h}))
{
	print <<"	Usage End.";
	Description:
		
		Version: $ver
		   v1.3: memory optimization.
		   v1.4: memory optimization.
		         delete -b bamfile option.
		   v1.5: memory optimization.

	Usage:

		-u           bc_umi_read.tsv.id file               <infile>     must be given
		-g           reads gene file(mult-line)            <infile>     must be given
		-o           result file                           <outfile>    must be given
		-h           Help document

	Usage End.

	exit;
}

###############Time
my $Time_Start;
$Time_Start = sub_format_datetime(localtime(time()));
&show_log("Start Time :[$Time_Start]");
################
$| = 1 ;
## get parameters
my $umifile = $opts{u} ;
my $genefile = $opts{g} ;
my $outfile = $opts{o} ;

## get reads umi info and out put
&get_umi_info_and_gene($umifile, $genefile, $outfile);

###############Time
my $Time_End;
$Time_End = sub_format_datetime(localtime(time()));
&show_log("End Time :[$Time_End]");

###############Subs
sub sub_format_datetime {#Time calculation subroutine
    my($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = @_;
	$wday = $yday = $isdst = 0;
    sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $day, $hour, $min, $sec);
}

sub ABSOLUTE_DIR
{ #$pavfile=&ABSOLUTE_DIR($pavfile);
	my $cur_dir=`pwd`;
	$cur_dir =~ s/\n$//;
	my ($in)=@_;
	my $return="";
	
	if(-f $in)
	{
		my $dir=dirname($in);
		my $file=basename($in);
		chdir $dir;$dir=`pwd`;
		$dir =~ s/\n$// ;
		$return="$dir/$file";
	}
	elsif(-d $in)
	{
		chdir $in;$return=`pwd`;
		$return =~ s/\n$// ;
	}
	else
	{
		warn "Warning just for file and dir [$in]\n";
		exit;
	}
	
	chdir $cur_dir;
	return $return;
}

# &show_log("txt")
sub show_log()
{
	my ($txt) = @_ ;
	my $time = time();
	my $Time = &sub_format_datetime(localtime($time));
	print "$Time:\t$txt\n" ;
	return ($time) ;
}

#&run_or_die($cmd);
sub run_or_die()
{
	my ($cmd) = @_ ;
	my $start_time = &show_log($cmd);
	my $flag = system($cmd) ;
	if ($flag != 0){
		my $end_time = &show_log("Error: command fail: $cmd");
		&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
		exit(1);
	}
	my $end_time = &show_log("done.");
	&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
	return ;
}

## qsub
sub qsub()
{
	my ($shfile, $maxproc) = @_ ;
	my $dir = dirname($shfile) ;
	my $cmd = "cd $dir && sh /share/nas2/genome/bmksoft/tool/qsub_sge_plus/v1.0/qsub_sge.plus.sh --maxproc $maxproc --reqsub --independent $shfile" ;
	&run_or_die($cmd);

	return ;
}

## qsub_mult($shfile, $max_proc, $job_num)
sub qsub_mult()
{
	my ($shfile, $max_proc, $job_num) = @_ ;
	if ($job_num > 500){
		my @shfiles = &cut_shfile($shfile);
		for my $file (@shfiles){
			&qsub($file, $max_proc);
		}
	}
	else{
		&qsub($shfile, $max_proc) ;
	}
}

#my @shfiles = &cut_shfile($shfile);
sub cut_shfile()
{
	my ($file) = @_ ;
	my @files = ();
	my $num = 0 ;
	open (IN, $file) || die "Can't open $file, $!\n" ;
	(my $outprefix = $file) =~ s/.sh$// ;
	while (<IN>){
		chomp ;
		if ($num % 500 == 0){
			close(OUT);
			my $outfile = "$outprefix.sub_".(int($num/500)+1).".sh" ;
			push @files, $outfile ;
			open (OUT, ">$outfile") || die "Can't creat $outfile, $!\n" ;
		}
		print OUT $_, "\n" ;
		$num ++ ;
	}
	close(IN);
	close(OUT);

	return @files ;
}

sub sub_normal_dis_ran(){#$average,$standard_deviation
        my ($aver,$stand_dev) = @_ ;
        my $ran1 = rand() ;
        my $ran2 = rand() ;
        my $y = ((-2*log($ran1))**(0.5))*sin(2*3.1415926535*$ran2) ;
        return ($aver + $stand_dev*$y) ;
}

#&get_align_reads($opts{b}, \%halign);
sub get_align_reads()
{
    my ($bamfile, $ahalign) = @_ ;

    open (IN, "samtools view $bamfile |") || die "$bamfile, $!\n" ;
    while(<IN>){
        chomp ;
        my ($id) = split ;
        $ahalign->{$id}++ ;
    }
    close(IN);

    return ;
}

#&get_gene_info($genefile, \%hrd2gene, \%hindex, \@all_genes);
sub get_gene_info()
{
    my ($genefile, $ahrd2gene, $ahindex, $aall_genes) = @_ ;

    open (IN, $genefile) || die "$genefile, $!\n" ;
    my $index = 0 ;
    while(<IN>){
        chomp ;
        next if (m/^\s*$|^\#/);
        my ($id, @genes) = split ;
        for my $gene (@genes){
            if (!defined $ahindex->{$gene}){
                push @{$aall_genes}, $gene ;
                $ahindex->{$gene} = $index ;
                $index++ ;
            }
            my $gene_index = $ahindex->{$gene} ;
            push @{$ahrd2gene->{$id}}, $gene_index ;
        }
    }
    close(IN);

    return ;
}

#&get_umi_info_and_gene($umifile, $genefile, $outfile);
sub get_umi_info_and_gene()
{
    my ($umifile, $genefile, $outfile) = @_ ;

    open (IN, $umifile) || die "$umifile, $!\n" ;
    open (OUT, ">$outfile") || die "$outfile, $!\n" ;
    my %hid = ();
    my %hbc = ();
    my $linenum = 0 ;
    while(<IN>){
        chomp ;
        next if (m/^\#|^\s*$/);
        $linenum++ ;
        my ($bc, $umi, @ids) = split ;
        $hbc{"$bc\t$umi"} = \@ids ;
        for my $id (@ids){
            $hid{$id} = 1 ;
        }
        if ($linenum % 25000000 == 0){
            &show_log("process $linenum...");
            open (GEN, $genefile) || die "$genefile, $!\n" ;
            my %hrd2gene = ();
            while(<GEN>){
                chomp ;
                next if (m/^\s*$|^\#/);
                my ($id, @genes) = split ;
                next if (!defined $hid{$id}) ;
                push @{$hrd2gene{$id}}, @genes ;
            }
            close(GEN);
            for my $type (keys %hbc){
                my $aids = $hbc{$type} ;
                my $gene = &get_bc_umi_gene(\%hrd2gene, $aids) ;
                next if ($gene eq "NULL") ;
                print OUT "$type\t$gene\n" ;
            }
            %hbc = ();
            %hid = ();
        }
    }
    open (GEN, $genefile) || die "$genefile, $!\n" ;
    my %hrd2gene = ();
    while(<GEN>){
        chomp ;
        next if (m/^\s*$|^\#/);
        my ($id, @genes) = split ;
        next if (!defined $hid{$id}) ;
        push @{$hrd2gene{$id}}, @genes ;
    }
    close(GEN);
    for my $type (keys %hbc){
        my $aids = $hbc{$type} ;
        my $gene = &get_bc_umi_gene(\%hrd2gene, $aids) ;
        next if ($gene eq "NULL") ;
        print OUT "$type\t$gene\n" ;
    }
    %hbc = ();
    %hid = ();
    close(IN);
    close(OUT);

    return ;
}

#my $gene = &get_bc_umi_gene(\%hrd2gene, $aids) ;
sub get_bc_umi_gene()
{
    my ($ahrd2gene, $aids) = @_ ;

    my %hstat = ();
    for my $id (@{$aids}){
        next if (!defined $ahrd2gene->{$id}) ;
        my @genes = @{$ahrd2gene->{$id}} ;
        my %huniq = ();
        for my $gene (@genes){
            $huniq{$gene}++ ;
        }
        for my $gene (keys %huniq){
            $hstat{$gene}++ ;
        }
    }
    my @genes = sort {$hstat{$b}<=>$hstat{$a}} keys %hstat ;
    if (scalar @genes > 0){
        my $gene = $genes[0] ;
        return($gene) ;
    }
    else{
        return("NULL") ;
    }
}

