#!/usr/bin/perl -w
# 
# Copyright (c) BIO_MARK 2021
# Writer:         Dengdj <dengdj@biomarker.com.cn>
# Program Date:   2021
# Modifier:       Dengdj <dengdj@biomarker.com.cn>
# Last Modified:  2021
my $ver="1.0";

use strict;
use Getopt::Long;
use Data::Dumper;
use FindBin qw($Bin $Script);
use File::Basename qw(basename dirname);

#####################

my %opts;
GetOptions(\%opts,"i=s","o=s","m=s","h" );

#&help()if(defined $opts{h});
if(!defined($opts{i}) || !defined($opts{o}) || defined($opts{h}))
{
	print <<"	Usage End.";
	Description:
		
		Version: $ver

	Usage:

		-i           read gene mult info file              <infile>     must be given
		-o           result file                           <outfile>    must be given
		-m           min map length                        [int]        optional [10]
		-h           Help document

	Usage End.

	exit;
}

###############Time
my $Time_Start;
$Time_Start = sub_format_datetime(localtime(time()));
&show_log("Start Time :[$Time_Start]");
################
$| = 1 ;
## get parameters
my $infile  = $opts{i} ;
my $outfile = $opts{o} ;
my $minmaplen = defined $opts{m} ? $opts{m} : 10 ;

## process
my %hreads = ();
my %hstat = ();
&get_reads_info_and_stat($infile, $minmaplen, $outfile, \%hreads, \%hstat);

## stat and output
&output_stat(\%hstat, $outfile);

###############Time
my $Time_End;
$Time_End = sub_format_datetime(localtime(time()));
&show_log("End Time :[$Time_End]");

###############Subs
sub sub_format_datetime {#Time calculation subroutine
    my($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = @_;
	$wday = $yday = $isdst = 0;
    sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $day, $hour, $min, $sec);
}

sub ABSOLUTE_DIR
{ #$pavfile=&ABSOLUTE_DIR($pavfile);
	my $cur_dir=`pwd`;
	$cur_dir =~ s/\n$//;
	my ($in)=@_;
	my $return="";
	
	if(-f $in)
	{
		my $dir=dirname($in);
		my $file=basename($in);
		chdir $dir;$dir=`pwd`;
		$dir =~ s/\n$// ;
		$return="$dir/$file";
	}
	elsif(-d $in)
	{
		chdir $in;$return=`pwd`;
		$return =~ s/\n$// ;
	}
	else
	{
		warn "Warning just for file and dir [$in]\n";
		exit;
	}
	
	chdir $cur_dir;
	return $return;
}

# &show_log("txt")
sub show_log()
{
	my ($txt) = @_ ;
	my $time = time();
	my $Time = &sub_format_datetime(localtime($time));
	print "$Time:\t$txt\n" ;
	return ($time) ;
}

#&run_or_die($cmd);
sub run_or_die()
{
	my ($cmd) = @_ ;
	my $start_time = &show_log($cmd);
	my $flag = system($cmd) ;
	if ($flag != 0){
		my $end_time = &show_log("Error: command fail: $cmd");
		&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
		exit(1);
	}
	my $end_time = &show_log("done.");
	&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
	return ;
}

## qsub
sub qsub()
{
	my ($shfile, $maxproc) = @_ ;
	my $dir = dirname($shfile) ;
	my $cmd = "cd $dir && sh /share/nas2/genome/bmksoft/tool/qsub_sge_plus/v1.0/qsub_sge.plus.sh --maxproc $maxproc --reqsub --independent $shfile" ;
	&run_or_die($cmd);

	return ;
}

## qsub_mult($shfile, $max_proc, $job_num)
sub qsub_mult()
{
	my ($shfile, $max_proc, $job_num) = @_ ;
	if ($job_num > 500){
		my @shfiles = &cut_shfile($shfile);
		for my $file (@shfiles){
			&qsub($file, $max_proc);
		}
	}
	else{
		&qsub($shfile, $max_proc) ;
	}
}

#my @shfiles = &cut_shfile($shfile);
sub cut_shfile()
{
	my ($file) = @_ ;
	my @files = ();
	my $num = 0 ;
	open (IN, $file) || die "Can't open $file, $!\n" ;
	(my $outprefix = $file) =~ s/.sh$// ;
	while (<IN>){
		chomp ;
		if ($num % 500 == 0){
			close(OUT);
			my $outfile = "$outprefix.sub_".(int($num/500)+1).".sh" ;
			push @files, $outfile ;
			open (OUT, ">$outfile") || die "Can't creat $outfile, $!\n" ;
		}
		print OUT $_, "\n" ;
		$num ++ ;
	}
	close(IN);
	close(OUT);

	return @files ;
}

sub sub_normal_dis_ran(){#$average,$standard_deviation
        my ($aver,$stand_dev) = @_ ;
        my $ran1 = rand() ;
        my $ran2 = rand() ;
        my $y = ((-2*log($ran1))**(0.5))*sin(2*3.1415926535*$ran2) ;
        return ($aver + $stand_dev*$y) ;
}

#&get_reads_info_and_stat($infile, $minmaplen, $outfile, \%hreads, \%hstat);
sub get_reads_info_and_stat()
{
    my ($infile, $minmaplen, $outfile, $ahreads, $ahstat) = @_ ;

    open (IN, $infile) || die "$infile, $!\n" ;
    open (OUT, ">$outfile.transcriptome") || die "$outfile.transcriptome, $!\n" ;
    open (EXN, ">$outfile.exon") || die "$outfile.exon, $!\n" ;
    while(<IN>){
        chomp ;
        next if (m/^\#|^\s*$/);
        my ($id, $region, $gene, $reg_map_len, $read_map_len, $chr, $start, $cigar, $NH, $HI, $AS, $nM) = split ;
        $ahstat->{mult}{$id}++ ;
        next if ($reg_map_len < $minmaplen);
        next if ($region ne 'exon');
        push @{$ahreads->{$id}}, [$gene, $chr, $reg_map_len];
    }
    close(IN);
    for my $id (keys %{$ahreads}){
        my %hgene = ();
        for (my $i=0; $i<@{$ahreads->{$id}}; $i++){
            my ($gene, $chr, $reg_map_len) = @{$ahreads->{$id}[$i]} ;
            $hgene{$gene}++ ;
        }
        my @genes = keys %hgene ;
        if (scalar @genes == 1){
            $ahstat->{Transcriptome}{$id}++ ;
            print OUT "$id\tTranscriptome\t$genes[0]\n" ;
        }
        else{
            print EXN "$id\tExon" ;
            for my $gene (@genes){
                print EXN "\t$gene" ;
            }
            print EXN "\n" ;
            $ahstat->{Exon}{$id}++ ;
        }
    }
    close(OUT);
    close(EXN);

    return ;
}

#&output_stat(\%hstat, $outfile);
sub output_stat()
{
    my ($ahstat, $outfile) = @_ ;

    open (OUT, ">$outfile.stat") || die "$outfile.stat, $!\n" ;
    for my $type (keys %{$ahstat}){
        my $num = scalar (keys %{$ahstat->{$type}});
        print OUT "$type\t$num\n" ;
    }
    close(OUT);

    return ;
}

